import 'package:flutter/material.dart';
import 'dart:async';

import 'package:flutter_webview_lite/flutter_webview_lite.dart';

void main() => runApp(MyApp());

class MyApp extends StatelessWidget {
  // This widget is the root of your application.
  @override
  Widget build(BuildContext context) {
    return MaterialApp(
      title: 'Flutter Demo',
      theme: ThemeData(
        // This is the theme of your application.
        //
        // Try running your application with "flutter run". You'll see the
        // application has a blue toolbar. Then, without quitting the app, try
        // changing the primarySwatch below to Colors.green and then invoke
        // "hot reload" (press "r" in the console where you ran "flutter run",
        // or simply save your changes to "hot reload" in a Flutter IDE).
        // Notice that the counter didn't reset back to zero; the application
        // is not restarted.
        primarySwatch: Colors.blue,
      ),
      home: LayoutDemo(),
    );
  }
}

class LayoutDemo extends StatefulWidget {
  @override
  _LayoutDemoState createState() => _LayoutDemoState();
}

class _LayoutDemoState extends State<LayoutDemo> {
  final flutterWebViewLite = FlutterWebViewLite();

  StreamSubscription _onSource;
  // On destroy stream
  StreamSubscription _onDestroy;

  // On urlChanged stream
  StreamSubscription<String> _onUrlChanged;

  // On urlChanged stream
  StreamSubscription<WebViewStateChanged> _onStateChanged;

  StreamSubscription<WebViewHttpError> _onHttpError;

  StreamSubscription<double> _onProgressChanged;

  StreamSubscription<double> _onScrollYChanged;

  StreamSubscription<double> _onScrollXChanged;

  final _scaffoldKey = GlobalKey<ScaffoldState>();

  final _history = [];

  void parseResult() {

  }

  @override
  void initState() {
    super.initState();
    flutterWebViewLite.close();

    _onSource = flutterWebViewLite.onSource.listen((String html) {
      if (mounted) {
        setState(() {
          _history.add('onSource: $html');
        });
      }
    });
// Add a listener to on destroy WebView, so you can make came actions.
    _onDestroy = flutterWebViewLite.onDestroy.listen((_) {
      if (mounted) {
        // Actions like show a info toast.
        _scaffoldKey.currentState.showSnackBar(
            const SnackBar(content: const Text('Webview Destroyed')));
      }
    });

    // Add a listener to on url changed
    _onUrlChanged = flutterWebViewLite.onUrlChanged.listen((String url) {
      if (mounted) {
        setState(() {
          _history.add('onUrlChanged: $url');
        });
      }
    });

    _onProgressChanged =
        flutterWebViewLite.onProgressChanged.listen((double progress) {
          if (mounted) {
            setState(() {
              _history.add("onProgressChanged: $progress");
            });
          }
        });

    _onScrollYChanged =
        flutterWebViewLite.onScrollYChanged.listen((double y) {
          if (mounted) {
            setState(() {
              _history.add('Scroll in Y Direction: $y');
            });
          }
        });

    _onScrollXChanged =
        flutterWebViewLite.onScrollXChanged.listen((double x) {
          if (mounted) {
            setState(() {
              _history.add('Scroll in X Direction: $x');
            });
          }
        });

    _onStateChanged =
        flutterWebViewLite.onStateChanged.listen((WebViewStateChanged state) {
          if (mounted) {
            setState(() {
              _history.add('onStateChanged: ${state.type} ${state.url}');
            });

            // state.type是一个枚举类型，取值有：WebViewState.shouldStart, WebViewState.startLoad, WebViewState.finishLoad
            switch (state.type) {
              case WebViewState.shouldStart:
              // 准备加载
                break;
              case WebViewState.startLoad:
              // 开始加载
                break;
              case WebViewState.finishLoad:
              // 加载完成
              // 当前是回调页面，则调用js方法获取数据
              // parseResult();
              // flutterWebViewPlugin.evalJavascript();
                break;
              case WebViewState.abortLoad:
                break;
            }
          }
        });

    _onHttpError =
        flutterWebViewLite.onHttpError.listen((WebViewHttpError error) {
          if (mounted) {
            setState(() {
              _history.add('onHttpError: ${error.code} ${error.url}');
            });
          }
        });
  }

  @override
  void dispose() {
    // Every listener should be canceled, the same should be done with this stream.
    _onDestroy.cancel();
    _onUrlChanged.cancel();
    _onStateChanged.cancel();
    _onHttpError.cancel();
    _onProgressChanged.cancel();
    _onScrollXChanged.cancel();
    _onScrollYChanged.cancel();
    _onSource.cancel();
    flutterWebViewLite.dispose();

    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: AppBar(
        title: Text('层叠定位布局'),
      ),
      body: SingleChildScrollView(
        child: Column(
          mainAxisAlignment: MainAxisAlignment.center,
          children: [
            RaisedButton(
              child: const Text('Open Webview (rect)'),
              onPressed: () {
                flutterWebViewLite.launch(
                  "https://m.quanben.net/273/273204/30383212.html",
                  hidden: true,
                  // rect: Rect.fromLTWH(0.0, 0.0, MediaQuery.of(context).size.width, 300.0),
                  // userAgent: kAndroidUserAgent,
                  // invalidUrlRegex: r'^(https).+(twitter)', // prevent redirecting to twitter when user click on its icon in flutter website
                );
              },
            ),
            Text(_history.join('\n'))
          ],
        ),
      ),
    );
  }
}

